unit antigate;

interface

uses Classes, SysUtils, IdMultiPartFormData, IdHTTP, StrUtils;

type TEvent = procedure(AMessage: string) of object;

type TAntiParam = record
  Key: string;
  Phrase: integer;
  Regsens: integer;
  Calc: integer;
  Numeric: integer;
  Russian: integer;
  MinLen: integer;
  MaxLen: integer;
  MaxRetry: integer;
  WaitTime: integer;
end;

type
  TAntiError = class
    private
    public
      const CAPTCHA_NOT_READY = 'CAPCHA_NOT_READY';
      const ERROR_NO_SLOT_AVAILABLE = 'ERROR_NO_SLOT_AVAILABLE';
      const ERROR_KEY_DOES_NOT_EXIST = 'ERROR_KEY_DOES_NOT_EXIST';
      const ERROR_ZERO_CAPTCHA_FILESIZE = 'ERROR_ZERO_CAPTCHA_FILESIZE';
      const ERROR_TOO_BIG_CAPTCHA_FILESIZE = 'ERROR_TOO_BIG_CAPTCHA_FILESIZE';
      const ERROR_WRONG_FILE_EXTENSION = 'ERROR_WRONG_FILE_EXTENSION';
      const ERROR_IP_NOT_ALLOWED = 'ERROR_IP_NOT_ALLOWED';
      const ERROR_WRONG_ID_FORMAT = 'ERROR_WRONG_ID_FORMAT';
      const ERROR_NO_SUCH_CAPCHA_ID = 'ERROR_NO_SUCH_CAPCHA_ID';
      const ERROR_URL_METHOD_FORBIDDEN = 'ERROR_URL_METHOD_FORBIDDEN';
      const OK_REPORT_RECORDED = 'OK_REPORT_RECORDED';
      const ERROR_HTTP_REQUEST = 'ERROR_HTTP_REQUEST';
      const ERROR_CAPTCHA_TIMEOUTED = 'ERROR_CAPTCHA_TIMEOUTED';
  end;

type
  TAntigate = class(TObject)
    private
      FHTTP: TIdHTTP;
      FStream: TMemoryStream;
      FFileName: string;
      FID: string;
      FKey: string;
      FPhrase: integer;
      FRegsens: integer;
      FCalc: integer;
      FNumeric: integer;
      FRussian: integer;
      FMinLen: integer;
      FMaxLen: integer;
      FMaxRetry: integer;
      FWaitTime: integer;
      FEvent: TEvent;

      procedure FSetAntiParam(Value: TAntiParam);
      function FGetAntiParam(): TAntiParam;

    public
      constructor Create; overload;
      constructor Create(AKey: string); overload;
      constructor Create(AParam: TAntiParam); overload;
      destructor Destroy; override;

      function Recognize: string;
      function GetBalance: string;

      procedure LoadFromFile(AFileName: string);
      procedure LoadFromStream(AFileName: string; AStream: TStream);

      procedure Abuse;

      property OnEvent: TEvent read FEvent write FEvent;
      property AntiParam: TAntiParam read FGetAntiParam write FSetAntiParam;
      property Key: string read FKey write FKey;
      property Phrase: integer read FPhrase write FPhrase;
      property Regsens: integer read FRegsens write FRegsens;
      property Calc: integer read FCalc write FCalc;
      property Numeric: integer read FNumeric write FNumeric;
      property Russian: integer read FRussian write FRussian;
      property MinLen: integer read FMinLen write FMinLen;
      property MaxLen: integer read FMaxLen write FMaxLen;
      property MaxRetry: integer read FMaxRetry write FMaxRetry;
      property WaitTime: integer read FWaitTime write FWaitTime;
  end;

implementation

{ TAntigate }

constructor TAntigate.Create;
begin
 Self.Create('');
end;

constructor TAntigate.Create(AKey: string);
begin
  inherited Create;
  FHTTP := TIdHTTP.Create;
  FStream := TMemoryStream.Create();
  FKey := AKey;
  FPhrase := 0;
  FRegsens := 0;
  FCalc := 0;
  FNumeric := 0;
  FRussian := 0;
  FMinLen := 0;
  FMaxLen := 0;
  FMaxRetry := 10;
  FWaitTime := 5000;
end;

constructor TAntigate.Create(AParam: TAntiParam);
begin
 Self.Create('');
 FSetAntiParam(AParam);
end;

destructor TAntigate.Destroy;
begin
  FStream.Free;
  FHTTP.Free;
  inherited Destroy;
end;

function TAntigate.FGetAntiParam: TAntiParam;
var
AntiParam: TAntiParam;
begin
AntiParam.Key := FKey;
AntiParam.Phrase := FPhrase;
AntiParam.Regsens := FRegsens;
AntiParam.Calc := FCalc;
AntiParam.Numeric := FNumeric;
AntiParam.Russian := FRussian;
AntiParam.MinLen := FMinLen;
AntiParam.MaxLen := FMaxLen;
AntiParam.MaxRetry := FMaxRetry;
AntiParam.WaitTime := FWaitTime;
Result := AntiParam;
end;

procedure TAntigate.FSetAntiParam(Value: TAntiParam);
begin
 FKey := Value.Key;
 FPhrase := Value.Phrase;
 FRegsens := Value.Regsens;
 FCalc := Value.Calc;
 FNumeric := Value.Numeric;
 FRussian := Value.Russian;
 FMinLen := Value.MinLen;
 FMaxLen := Value.MaxLen;
 FMaxRetry := Value.MaxRetry;
 FWaitTime := Value.WaitTime;
end;

procedure TAntigate.LoadFromFile(AFileName: string);
begin
  FFileName := AFileName;
  FStream.LoadFromFile(AFileName);
end;

procedure TAntigate.LoadFromStream(AFileName: string; AStream: TStream);
begin
  FFileName := AFileName;
  FStream.LoadFromStream(AStream);
end;

function TAntigate.GetBalance: string;
var
  AnswerResult: string;
begin
  try
    FHTTP.Request.ContentType := '';
    AnswerResult := FHTTP.Get('http://antigate.com/res.php?key=' + FKey + '&action=getbalance');
  except
    if Assigned(FEvent) then FEvent(TAntiError.ERROR_HTTP_REQUEST);
  end;
  Result := AnswerResult;
end;

function TAntigate.Recognize: string;
var
  MultiPart: TIdMultiPartFormDataStream;
  AnswerResult: string;
  RetryCount: integer;
begin
  Result := '';
  MultiPart := TIdMultiPartFormDataStream.Create;
  MultiPart.AddFormField('method', 'post');
  MultiPart.AddFormField('key', FKey);
  if FPhrase <> 0 then MultiPart.AddFormField('phrase', IntToStr(FPhrase));
  if FRegsens <> 0 then MultiPart.AddFormField('regsense', IntToStr(FRegsens));
  if FCalc <> 0 then MultiPart.AddFormField('calc', IntToStr(FCalc));
  if FNumeric <> 0 then MultiPart.AddFormField('numeric', IntToStr(FNumeric));
  if FRussian <> 0 then MultiPart.AddFormField('is_russian', IntToStr(FRussian));
  if FMinLen <> 0 then MultiPart.AddFormField('min_len', IntToStr(FMinLen));
  if FMaxLen <> 0 then MultiPart.AddFormField('max_len', IntToStr(FMaxLen));
  MultiPart.AddObject('file', 'application/octet-stream', FStream, FFileName);
  while (true) do
  begin
    try
      FHTTP.Request.ContentType := '';
      AnswerResult := FHTTP.Post('http://antigate.com/in.php', MultiPart);
    except
      if Assigned(FEvent) then FEvent(TAntiError.ERROR_HTTP_REQUEST);
      exit;
    end;
    if Pos('|', AnswerResult) > 0 then
    begin
      MultiPart.Free;
      FID := AnsiReplaceStr(AnswerResult, 'OK|', '');
      RetryCount := 0;
      while (true) do
      begin
        if RetryCount > FMaxRetry then
        begin
          if Assigned(FEvent) then FEvent(TAntiError.ERROR_CAPTCHA_TIMEOUTED);
          exit;
        end;
        Sleep(FWaitTime);
        try
          FHTTP.Request.ContentType := '';
          AnswerResult := FHTTP.Get('http://antigate.com/res.php?key=' + FKey + '&action=get&id=' + FID);
        except
          if Assigned(FEvent) then FEvent(TAntiError.ERROR_HTTP_REQUEST);
          exit;
        end;
        if Pos('|', AnswerResult) > 0 then
        begin
          Result := AnsiReplaceStr(AnswerResult, 'OK|', '');
          exit;
        end
        else
        begin
          if Assigned(FEvent) then FEvent(AnswerResult);
          if AnswerResult = TAntiError.CAPTCHA_NOT_READY then
          Inc(RetryCount) else exit;
        end;
      end;
    end
    else if AnswerResult = TAntiError.ERROR_NO_SLOT_AVAILABLE then
    begin
    if Assigned(FEvent) then FEvent(AnswerResult);
    Sleep(5000);
    end
    else
    begin
      if Assigned(FEvent) then FEvent(AnswerResult);
      exit;
    end;
  end;
end;

procedure TAntigate.Abuse;
begin
  try
    FHTTP.Request.ContentType := '';
    FHTTP.Get('http://antigate.com/res.php?key=' + FKey + '&action=reportbad&id=' + FID);
  except
    if Assigned(FEvent) then FEvent(TAntiError.ERROR_HTTP_REQUEST);
    exit;
  end;
end;

end.
